from string import Template
from typing import Optional, Set


class JnodeError(ValueError):
    pass


class BufferSizeError(BufferError):
    def __init__(self, min_size: int):
        self.min_size = min_size


class ImageRef:
    def __init__(self,
                 ts: Optional[int] = None,
                 url: Optional[str] = None,
                 codec: Optional[str] = None,
                 width: Optional[int] = None,
                 height: Optional[int] = None):
        self.ts = ts
        self.url = url
        self.codec = codec
        self.width = width
        self.height = height


class FormatTemplate(Template):
    idpattern = r'[_a-z][-_a-z0-9]*'

    _KNOWN_PHOLDERS: Set[str] = {
        'datetime',
        'datetime-ms',
        'datetime-control',
        'datetime-control-ms',
        'time-control-ms',
        'post-name',
        'geolocation',
        'speed',
        'plate',
        'serial',
        'lane',
        'dir',
        'video-speed',
        'video-speed2',
        'radar-speed',
        'radar-speed2',
        'max-speed',
        'speed-limit',
        'radar-overspeed',
        'model',
        'vendor',
        'section-speed',
        'section-overspeed',
        'wim-speed',
        'main-speed',
        'control-speed',
        'main-overspeed',
        'wim-overspeed',
        'halt-time'
    }

    @staticmethod
    def get_pholders(string_format: str) -> Set[str]:
        """
        :raise: ValueError
        """
        template = FormatTemplate(string_format)

        if not template.is_valid():
            raise ValueError

        string_set = set(template.get_identifiers())
        if not string_set.issubset(FormatTemplate._KNOWN_PHOLDERS):
            raise ValueError

        return string_set
