from typing import Self, Union

from edge import eventlog
from edge.exceptions import NotFoundError


class UiRef:
    def __init__(self, ref: str):
        assert ref
        self._ref = ref

    def __eq__(self, other) -> bool:
        return isinstance(other, self.__class__) and getattr(other, 'ref', None) == self.ref

    def __hash__(self) -> int:
        return hash(self.ref)

    def __str__(self) -> str:
        return self.ref

    @property
    def ref(self) -> str:

        return self._ref

    @property
    def type(self) -> str:
        segments = self.ref.split('~')
        subtypes = [x.split('.')[0] for x in segments]
        return '~'.join(subtypes)

    def assert_type(self, other_type: str) -> None:
        """
        :raises NotFoundError:
        """
        if self.type != other_type:
            eventlog.log_w(f'fail: NotFoundError ({self.ref})')
            raise NotFoundError

    @property
    def root_ref(self) -> str:
        """
        :raises NotFoundError:
        """
        return self.ref.split('~', 1)[0]

    @property
    def parent_ref(self) -> str:
        """
        :raises NotFoundError:
        """
        parent_ref = '~'.join(self.ref.split('~')[:-1])
        if not parent_ref:
            raise NotFoundError

        return parent_ref

    @property
    def parent(self) -> Self:
        """
        :rtype: UiRef
        :raises NotFoundError:
        """
        return UiRef(self.parent_ref)

    def child_ref(self, child_type: str, child_chan: Union[str, int]) -> str:
        if not child_type.startswith(self.type):
            raise NotFoundError

        child_type_segments = child_type.split('~')
        if not len(self.type.split('~')) == len(child_type_segments) - 1:
            raise NotFoundError

        return f'{self.ref}~{child_type_segments[-1]}.{child_chan}'

    def child(self, child_type: str, child_chan: Union[str, int]) -> Self:
        return UiRef(self.child_ref(child_type, child_chan))
