from typing import Dict, Self

from edge import eventlog
from edge.image.jnode.types import JsonImEngineArgs
from edge.types import JnodeError

from speedcontrol.detectors.base.types import DetectorArgs


class JsonDetectorArgs(DetectorArgs):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @classmethod
    def deserialize(cls, jnode: Dict, def_footer_mapping: Dict) -> Self:
        """
        :raise JnodeError:
        """
        # deserialize #/images
        images_jnode = jnode.get('images')
        if type(images_jnode) != dict:
            eventlog.log_e('fail: #/images')
            raise JnodeError

        # deserialize #/images/best
        best_waypoint_jnode = images_jnode.get('best')
        if type(best_waypoint_jnode) != dict:
            eventlog.log_e_param('#/images/best')
            raise JnodeError

        im_engine = JsonImEngineArgs.deserialize(best_waypoint_jnode)

        # deserialize #/systest
        systest = jnode.get('systest', False)

        # deserialize #/enable-overview
        enable_overview = jnode.get('enable-overview', False)

        return cls(def_footer_mapping, im_engine, systest, enable_overview)
