import datetime

from typing import Dict, List, Optional

from speedcontrol.model.types import Ticket


class ScheduleArgs:
    def __init__(self, min_time: datetime.time, max_time: datetime.time, days: List[bool]):
        self.min_time = min_time
        self.max_time = max_time
        self.days_to_bool = days

    def serialize(self) -> Dict:
        jnode = {
            "min-time": [
                self.min_time.hour,
                self.min_time.minute
            ],
            "max-time": [
                self.max_time.hour,
                self.max_time.minute
            ],
            "days": {
                "mo": self.days_to_bool[0],
                "tu": self.days_to_bool[1],
                "we": self.days_to_bool[2],
                "th": self.days_to_bool[3],
                "fr": self.days_to_bool[4],
                "sa": self.days_to_bool[5],
                "su": self.days_to_bool[6]
            }
        }

        return jnode


class BaseRuleArgs:
    def __init__(self, name: str, enabled: bool, ticket: Ticket, schedule: Optional[ScheduleArgs] = None):
        self.name = name
        self.enabled = enabled
        self.ticket = ticket
        self.schedule = schedule


class VideoArgs:
    def __init__(self,
                 max_w: int = 32752,
                 max_h: int = 32752,
                 video_format: str = 'h264-mp4',
                 quality: int = 500,
                 pre_record: int = 5000,
                 post_record: int = 5000) -> object:

        self.max_w = max_w
        self.max_h = max_h
        self.pre_record = pre_record
        self.post_record = post_record

        self.quality = quality
        self.format = video_format
