from typing import Dict, List, Optional

from edge.image.types import ImEngineArgs
from edge.model.types import Origin
from edge.service.tz import ITzService

from speedcontrol.detectors.com.types import ScheduleArgs
from speedcontrol.model.types import Identity, Evidence, SceneArgs, Ticket


class RuleLimits:
    def __init__(self, max_standing_time: int):
        self.max_standing_time = max_standing_time


class RuleArgs:
    def __init__(self, name: str, enabled: bool, schedule: Optional[ScheduleArgs], ticket: Ticket, limits: RuleLimits):
        self.name = name
        self.enabled = enabled
        self.schedule = schedule
        self.ticket = ticket
        self.limits = limits


class DetectorArgs:
    def __init__(self,
                 identity: Identity,
                 name: str,
                 store: str,
                 scene: SceneArgs,
                 rules: List[RuleArgs],
                 init_im_engine: ImEngineArgs,
                 fini_im_engine: ImEngineArgs,
                 def_footer_mapping: Dict,
                 range_tout: int,
                 lost_tout: int):

        self.identity = identity
        self.name = name
        self.store = store
        self.scene = scene
        self.rules = rules
        self.init_im_engine = init_im_engine
        self.fini_im_engine = fini_im_engine
        self.def_footer_mapping = def_footer_mapping
        self.range_tout = range_tout * 1e6
        self.lost_tout = lost_tout * 1e6


class ParkingEvidence(Evidence):
    def __init__(self, detector_name: str, rule_args: RuleArgs, ts: int, origin: Origin, standing_time: int):
        self._detector_name = detector_name
        self._standing_time = standing_time
        self.rule_args = rule_args

        super().__init__(Evidence.NAME_PARKING, origin, ts)

    @property
    def standing_time(self) -> int:
        return self._standing_time

    def serialize(self, tz_service: ITzService) -> Dict:
        jnode = Evidence.serialize(self, tz_service)
        waypoints_jnode = [x.serialize(tz_service) for x in self._waypoints]
        jnode['evidence'] = {
            'detector': {
                'name': self._detector_name,
                'rule': {
                    'name': self.rule_args.name,
                    'schedule': self.rule_args.schedule.serialize(),
                    'ticket': {
                        'name': self.rule_args.ticket.name
                    },
                    'limits': {
                        'max-standing-time': self.rule_args.limits.max_standing_time
                    }
                }
            },
            'measurements': {
                'standing-time': self._standing_time,
                'waypoints': {
                    'violation': waypoints_jnode
                }
            }
        }

        return jnode
