from typing import Dict, Self

from edge import eventlog
from edge.types import JnodeError

from speedcontrol.detectors.com.jnode.types import JsonScheduleArgs
from speedcontrol.detectors.transit.types import DetectorArgs, ImagesArgs
from speedcontrol.detectors.types import IDetector
from speedcontrol.model.types import Identity


class JsonImagesArgs(ImagesArgs):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @classmethod
    def deserialize(cls, args_jnode: Dict) -> Self:
        """
        :raise JnodeError:
        """
        def deserialize_store(im_profile):
            profile_jnode = args_jnode.get(im_profile)
            if dict != type(profile_jnode):
                eventlog.log_e_param('#%s' % im_profile)
                raise JnodeError

            store = profile_jnode.get('store')
            if not (type(store) in [str] and 1 <= len(store) <= 8):
                eventlog.log_e_param('#/%s/store' % im_profile)
                raise JnodeError

            return store

        scene_store, vehicle_store = list(map(
            deserialize_store, ['scene', 'vehicle']))

        return cls(vehicle_store, scene_store)


class JsonDetectorArgs(DetectorArgs):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @classmethod
    def deserialize(cls, transit_jnode: Dict) -> Self:
        rules_jnode = transit_jnode['rules']

        # deserialize #/rules
        if type(rules_jnode) != list or len(rules_jnode) != 1:
            eventlog.log_e_param('#/rules')
            raise JnodeError

        # deserialize #/rules[0]/schedule
        schedule_args = None
        schedule_jnode = rules_jnode[0].get('schedule')
        if schedule_jnode is not None:
            if dict != type(schedule_jnode):
                eventlog.log_e_param('#/schedule')
                raise JnodeError

            schedule_args = JsonScheduleArgs.deserialize(schedule_jnode)

        # deserialize #/rules[0]/name
        name = rules_jnode[0].get('name')
        if type(name) not in [str]:
            eventlog.log_e_param('#/name')
            raise JnodeError

        # deserialize #/rules[0]/enabled
        enabled = rules_jnode[0].get('enabled')
        if type(enabled) != bool:
            eventlog.log_e_param('#/enabled')
            raise JnodeError

        # deserialize #/rules[0]/limits
        limits_jnode = rules_jnode[0].get('limits')
        if type(limits_jnode) != dict:
            eventlog.log_e_param('#/limits')
            raise JnodeError

        require_plate = limits_jnode.get('require-plate', True)
        if type(require_plate) != bool:
            eventlog.log_e_param('#/limits/require-plate')
            raise JnodeError

        # deserialize #/images
        images_jnode = transit_jnode.get('images')
        if type(images_jnode) != dict:
            eventlog.log_e_param('#/images')
            raise JnodeError

        # deserialize #/images/best
        best_waypoint_jnode = images_jnode.get('best')
        if type(best_waypoint_jnode) != dict:
            eventlog.log_e_param('#/images/best')
            raise JnodeError

        images = JsonImagesArgs.deserialize(best_waypoint_jnode)

        identity = Identity(IDetector.TYPE_TRANSIT, 0)
        plate_detected_mode = transit_jnode.get('plate-detected-mode', False)
        le_loop_ref = transit_jnode.get('le-loop-ref', None)
        return cls(name, enabled, require_plate, schedule_args, images, identity,
                   plate_detected_mode, le_loop_ref)
