from typing import Dict, List, Optional

from edge.image.types import MemBlob
from edge.model.types import Origin
from edge.service.tz import ITzService

from speedcontrol.detectors.com.types import ScheduleArgs
from speedcontrol.model.types import Identity, Evidence


class ImagesArgs:
    def __init__(self, vehicle_store: str, scene_store: str):
        self.vehicle_store = vehicle_store
        self.scene_store = scene_store


class DetectorArgs:
    def __init__(self,
                 name: str,
                 enabled: bool,
                 require_plate: bool,
                 schedule_args: Optional[ScheduleArgs],
                 blob_store_args: ImagesArgs,
                 identity: Identity,
                 plate_detected_mode: bool = False,
                 le_loop_ref: Optional[str] = 'loop0'):

        self.name = name
        self.enabled = enabled
        self.require_plate = require_plate
        self.schedule_args = schedule_args
        self.blob_store_args = blob_store_args
        self.identity = identity
        self.plate_detected_mode = plate_detected_mode
        self.le_loop_ref = le_loop_ref


class TransitEvidence(Evidence):
    def __init__(self, origin: Origin, ts: int, control_ts: Optional[int] = None):
        super().__init__(Evidence.NAME_TRANSIT, origin, ts, control_ts=control_ts)

    def get_blobs(self) -> List[MemBlob]:
        return []

    def serialize(self, tz_service: ITzService) -> Dict:
        jnode = Evidence.serialize(self, tz_service)
        return jnode
