from typing import Callable, Dict

from speedcontrol.model.models import Track
from speedcontrol.model.types import BaseEvent


class DetectorError(RuntimeError):
    pass


class IBaseDetector:

    def set_handler(self, event_handler: Callable[[Track, BaseEvent], None]) -> None:
        raise NotImplementedError

    def analyze(self, track: Track) -> None:
        raise NotImplementedError

    def destroy(self) -> None:
        pass

    def collect_stats(self, json_handler: Callable[[Dict], None]) -> None:
        pass


class IDetector:
    TYPE_TRANSIT = 0
    TYPE_PARKING = 1
    TYPE_WRONG_WAY = 2
    TYPE_SPOT_SPEED = 3
    TYPE_RED_LIGHT = 4
    TYPE_WANTED = 5
    TYPE_SECTION_SPEED = 6
    TYPE_BUS_LANE = 7

    def set_handler(self, event_handler: Callable[[Track, BaseEvent], None]) -> None:
        raise NotImplementedError

    def analyze(self, track: Track, base: BaseEvent) -> None:
        raise NotImplementedError

    def collect_stats(self, json_handler: Callable[[Dict], None]) -> None:
        pass

    def destroy(self) -> None:
        pass
