from edge import eventlog
from edge.types import JnodeError

from speedcontrol.detectors.com.jnode.types import JsonBaseRuleArgs
from speedcontrol.detectors.types import IDetector
from speedcontrol.detectors.wanted.types import DetectorArgs, RuleArgs, LimitsArgs
from speedcontrol.model.types import Identity


class JsonLimitsArgs(LimitsArgs):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @staticmethod
    def deserialize(jnode):
        """
        :param jnode: wanted.conf#/rules/:x/limits
        :type jnode: dict
        :rtype: JsonLimitsArgs
        :raise JsonError:
        """
        # deserialize #/plate-dist
        plate_dist = jnode.get('plate-dist', 0)
        if type(plate_dist) not in [int]:
            eventlog.log_e_param('#/plate-dist')
            raise JnodeError

        # deserialize #/plate-list
        list_jnode = jnode.get('plate-list')
        if type(list_jnode) != dict:
            eventlog.log_e_param('#/plate-list')
            raise JnodeError

        # deserialize #/plate-list/owner
        list_owner = list_jnode.get('owner')
        if not type(list_owner) in [str] or len(list_owner) > 32:
            eventlog.log_e_param('#/plate-list/owner')
            raise JnodeError

        # deserialize #/plate-list/name
        list_name = list_jnode.get('name')
        if not type(list_name) in [str] or len(list_name) > 32:
            eventlog.log_e_param('#/plate-list/name')
            raise JnodeError

        return JsonLimitsArgs(plate_dist, list_owner, list_name)


class JsonRuleArgs(RuleArgs):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @staticmethod
    def deserialize(jnode):
        """
        :param jnode: wanted.conf#/rules/:x
        :type jnode: dict
        :rtype: JsonRuleArgs
        :raise JsonError:
        """
        if type(jnode) != dict:
            eventlog.log_e_param('#/')
            raise JnodeError

        # deserialize #/name
        # deserialize #/enabled
        # deserialize #/ticket
        rule = JsonBaseRuleArgs.deserialize(jnode)

        # deserialize #/limits
        limits_jnode = jnode.get('limits')
        if type(jnode) != dict:
            eventlog.log_e_param('#/limits')
            raise JnodeError

        limits = JsonLimitsArgs.deserialize(limits_jnode)

        return JsonRuleArgs(
            rule.name, rule.enabled, rule.ticket,
            limits)


class JsonDetectorArgs(DetectorArgs):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @staticmethod
    def deserialize(jnode, index, def_footer_mapping):
        """
        :param jnode: wanted.conf#/
        :type jnode: dict
        :type index: int
        :type def_footer_mapping: dict
        :rtype: JsonDetectorArgs
        :raise JsonError:
        """
        # deserialize #/name
        name = jnode.get('name')
        if not type(name) in [str] or len(name) > 32:
            eventlog.log_e_param('#/name')
            raise JnodeError

        # deserialize #/store
        store = jnode.get('store')
        if not (type(store) in [str] and 1 <= len(store) <= 2):
            eventlog.log_e_param('#/store')
            raise JnodeError

        # deserialize #/rules
        rules_jnode = jnode.get('rules')
        if type(rules_jnode) != list:
            eventlog.log_e_param('#/rules')
            raise JnodeError

        rules = list(map(JsonRuleArgs.deserialize, rules_jnode))
        identity = Identity(IDetector.TYPE_WANTED, index)

        return JsonDetectorArgs(identity, name, store, rules)
