from typing import Dict

from edge.com import fs

from speedcontrol.services.types import SystestServiceArgs


class ISystestService:
    def serialize(self, jnode: Dict) -> Dict:
        raise NotImplementedError


class _StubSystestService(ISystestService):
    def __init__(self):
        ISystestService.__init__(self)

    def serialize(self, jnode: Dict) -> Dict:
        return jnode


class _SystestService(ISystestService):
    def __init__(self, args: SystestServiceArgs):
        ISystestService.__init__(self)

        jnode = fs.load_json3(args.conf_path, default=dict())
        rw_jnode = jnode.get('rw') or dict()
        self._crt_jnode = rw_jnode.get('certificate')

    def serialize(self, jnode: Dict) -> Dict:
        if self._crt_jnode:
            jnode.update({'certificate': self._crt_jnode})

        return jnode


def init_service(args: SystestServiceArgs) -> ISystestService:
    if args.conf_path:
        systest = _SystestService(args)
    else:
        systest = _StubSystestService()

    return systest


# noinspection PyUnusedLocal
def fini_service(service: ISystestService):
    pass
