import datetime

from typing import List, Optional

from edge.com.db import DbArgs
from edge.lane_detector import LanesConfStr
from edge.model.types import Location


class TlServiceArgs:
    def __init__(self, endpoint: str, tl_ident: int, red_only: bool = False):
        self.endpoint = endpoint
        self.tl_ident = tl_ident
        self.red_only = red_only


class LeddarServiceArgs:
    def __init__(self, endpoint: str, ref: str):
        self.endpoint = endpoint
        self.ref = ref


class VsServiceArgs:
    def __init__(self, store: int, tmp_path: str, chan: str = '0', log_level: str = 'info'):
        self.store = store
        self.tmp_path = tmp_path
        self.chan = chan
        self.log_level = log_level


class BsServiceArgs:
    def __init__(self, module_id: int, sender_to_chan_map, log_level: str):
        """
        :type module_id: int
        :type sender_to_chan_map: dict [Identity, str | dict [str, str]]
        :type log_level: str
        """
        self.module_id = module_id
        self.sender_to_chan_map = sender_to_chan_map
        self.log_level = log_level


class AboutServiceArgs:
    def __init__(self, uuid: str, time: str, location: Location):
        self.uuid = uuid
        self.time = time
        self.location = location


class RaasServiceArgs:
    def __init__(self, raas_host: str, raas_port: str, raas_user: str, raas_pass: str):
        self.raas_host = raas_host
        self.raas_port = raas_port
        self.raas_user = raas_user
        self.raas_pass = raas_pass


class EventQueueArgs:
    def __init__(self, endpoint: str, min_file_count: int, max_file_count: int):
        self.endpoint = endpoint
        self.min_file_count = min_file_count
        self.max_file_count = max_file_count


class EventServiceArgs:
    def __init__(self,
                 require_plate: bool,
                 json_queue_list: List[EventQueueArgs],
                 ua_json_required: bool,
                 local_arch_base_dir: Optional[str]):

        self.require_plate = require_plate
        self.json_queue_list = json_queue_list
        self.ua_json_required = ua_json_required

        self.local_arch_enabled = local_arch_base_dir is not None
        self.local_arch_base_dir = local_arch_base_dir


class ReactServiceArgs:
    def __init__(self, react_endpoint: str):
        self.react_endpoint = react_endpoint


class TrainerServiceArgs:
    LR_ANY = 'any'
    LR_PRESRNT = 'present'
    LR_ABSENT = 'absent'

    def __init__(self,
                 root_dir: str,
                 lanes: List[str],
                 min_time: datetime.time,
                 max_time: datetime.time,
                 min_df: float,
                 lr: str = LR_ANY):

        self.root_dir = root_dir
        self.min_time = min_time
        self.max_time = max_time
        self.lanes = lanes
        self.min_df = min_df
        self.lr = lr


class StatServiceArgs:
    def __init__(self, stat_filepath: str, stat_tout_sec: int):
        self.stat_filepath = stat_filepath
        self.stat_tout_sec = stat_tout_sec


class LaneDetectorServiceArgs:
    def __init__(self,
                 id_to_lane_list: LanesConfStr,
                 im_width: int,
                 im_height: int,
                 approaching: str,
                 receding: str,
                 unknown: str):

        self.id_to_lane_list = id_to_lane_list
        self.img_width = im_width
        self.img_height = im_height
        self.approaching = approaching
        self.receding = receding
        self.unknown = unknown


class DbServiceArgs:
    def __init__(self, origin_id: int, db_args: DbArgs):
        self.origin_id = origin_id
        self.db_args = db_args


class SignServiceArgs:
    def __init__(self, iit_conf: str, log_level: str = 'info'):
        self.iit_conf = iit_conf
        self.log_level = log_level


class SystestServiceArgs:
    def __init__(self, conf_path: Optional[str]):
        self.conf_path = conf_path


class PlateInfoServiceConf:
    def __init__(self, info_path: Optional[str]):
        self.info_path = info_path
